@@ -1,4 +1,5 @@
 from snowflake.ml.experiment._entities.experiment import Experiment
 from snowflake.ml.experiment._entities.run import Run
+from snowflake.ml.experiment._entities.run_metadata import Metric, Param
 
-__all__ = ["Experiment", "Run"]
+__all__ = ["Experiment", "Run", "Metric", "Param"]
